use hittingset::{DefaultEdge, Edge, search};
use plistore::{PLITable, RowIndex};

use crate::edgecollectors::{EdgeConsumer, EdgeHashCollector};
use crate::problems::common::get_default_edge;
use crate::problems::ProblemConfig;
use crate::sampling::{SamplingStrategy, SimpleSampling};
use crate::validation::PLIValidator;

pub struct UCCProblem<'a> {
    table: &'a PLITable,
}

impl<'a> UCCProblem<'a> {
    pub fn new(table: &'a PLITable) -> UCCProblem<'a> { UCCProblem { table } }
}

impl<'a> ProblemConfig for UCCProblem<'a> {
    type EdgeType = DefaultEdge;

    fn get_edge(&self, p1: RowIndex, p2: RowIndex, receiver: &mut impl EdgeConsumer<Self::EdgeType>) {
        receiver.consume(get_default_edge(self.table, p1, p2));
    }

    fn is_valid_result(&self, _result: &Self::EdgeType) -> bool {
        true
    }

    fn is_violating_pair(&self, _r1: RowIndex, _r2: RowIndex, _candidate: &Self::EdgeType) -> bool {
        true
    }
}

pub fn search_uccs(table: &PLITable, sampling_factor: f64, consumer: &mut impl EdgeConsumer<DefaultEdge>) {
    let problem = UCCProblem {
        table
    };
    let sampling = SimpleSampling::new(sampling_factor, &problem);
    let mut collector = EdgeHashCollector::new();
    for c in 0..table.column_count() {
        sampling.sample_pli(table.get_column_pli(c), &mut collector);
    }
    let mut input = collector.to_graph(&DefaultEdge::empty(table.column_count()));

    let mut validator = PLIValidator::new(table, consumer, &problem, &sampling);
    search(&mut input, &mut validator);
}
