use hittingset::{Edge, Hypergraph, Validator};
use crate::treesearch::dynamicedgemarkstoremulti::DynamicEdgemarkStoreMulti;
use crate::problems::results::MultiFD;

pub fn search(partial: &mut Hypergraph<MultiFD>, target: &mut impl Validator<MultiFD>) {
    let vertex_count = partial.vertex_count();
    let mut s = MultiFD::empty(vertex_count);

    let mut store = DynamicEdgemarkStoreMulti::new(partial);

    extend_or_confirm(&mut s, &mut store, target, 0);
}

fn extend_or_confirm<'a>(s: &mut MultiFD, store: &mut DynamicEdgemarkStoreMulti<'a>, target: &mut impl Validator<MultiFD>, level: usize) {
    if store.get_current_cand().rhs.fast_nodes().next().is_none() {
        return;
    }

    target.push(store.get_nodes());

    if store.is_fully_covered() {
        s.rhs.set_to(&store.get_current_cand().rhs);
        // println!("{} Result: {}", " ".repeat(level), s);
        let (invalid,  edges) = target.check(s, store.get_nodes(), store.get_cand());

        if let Some(new_edges) = edges {
            store.extend(s, new_edges);
        }

        if !invalid {
            return
        }
    }

    // find edge from uncov with smallest intersection c with cand
    let c = get_best_edge(store);
    // println!("{} Split on edge {:}"," ".repeat(level), c);

    let check_rhs_first = false;

    if check_rhs_first {
        store.get_current_cand_mut().lhs.remove_all(&c.lhs);
    }

    if c.rhs.fast_nodes().next().is_some() && store.add_rhs(&c.rhs) {
        // println!("{} Candidates RHS: {} for {} with {}"," ".repeat(level), store.get_current_cand(), c, s);
        extend_or_confirm(s, store, target, level + 1);
        store.pop();
    }

    if !check_rhs_first {
        store.get_current_cand_mut().remove_all(&c);
    }

    for v in c.lhs.fast_nodes() {
        if store.add_lhs(v) {
            s.add(v);

            // println!("{} Candidates LHS: {} for {} with {}", " ".repeat(level), store.get_current_cand(), c, s);
            extend_or_confirm(s, store, target, level + 1);

            s.remove(v);

            store.pop();
        }
    }
}

#[inline]
fn get_best_edge<'a>(edgemarkstore: &'a DynamicEdgemarkStoreMulti<'a>) -> MultiFD {
    edgemarkstore.uncovered_edges().min_by_key(|c| c.overlap(edgemarkstore.get_current_cand())).unwrap().intersect(edgemarkstore.get_current_cand())
}