use std::marker::PhantomData;
use hittingset::Edge;
use plistore::{Cluster, ClusterFilter};
use crate::edgecollectors::EdgeConsumer;
use crate::sampling::SamplingStrategy;

pub(crate) struct SmallClusterCollector<'a, E : Edge, S: SamplingStrategy<E>, C: EdgeConsumer<E>> {
    sampling: &'a S,
    consumer: &'a mut C,
    limit: u64,
    requested_level: bool,
    require_requested_level: bool,
    phantom: PhantomData<E>
}

impl<'a, E : Edge, S: SamplingStrategy<E>, C: EdgeConsumer<E>> SmallClusterCollector<'a, E, S, C> {
    pub(crate) fn new(sampling: &'a S, consumer: &'a mut C, limit: u64, require_requested_level: bool) -> SmallClusterCollector<'a, E, S, C> {
        SmallClusterCollector {
            sampling,
            consumer,
            limit,
            requested_level: false,
            require_requested_level,
            phantom: Default::default()
        }
    }
}

impl<'a, E : Edge, S: SamplingStrategy<E>, C: EdgeConsumer<E>> ClusterFilter for SmallClusterCollector<'a, E, S, C> {
    fn keep(&mut self, cluster: &Cluster) -> bool {
        if (!self.require_requested_level || self.requested_level) && self.sampling.get_cluster_size(cluster) < self.limit {
            self.sampling.get_cross_product(cluster, self.consumer);
            return false;
        }
        true
    }

    fn set_requested_level(&mut self, active: bool) {
        self.requested_level = active;
    }
}
