use std::fs::File;
use criterion::{black_box, criterion_group, criterion_main, Criterion};
use hittingset::{DynamicEdgemarkStore, DefaultEdge, StoreOps, Edge};

fn bench(test: &Vec<StoreOps<DefaultEdge>>) -> usize {
    let mut iter = test.iter();
    let mut graph = match iter.next().unwrap() {
        StoreOps::New(x) => {
            x.to_owned()
        },
        _ => panic!("No new graph")
    };
    let mut t = DynamicEdgemarkStore::new(&mut graph);
    let  mut size = 0;
    for e in iter {
        match e {
            StoreOps::New(x) => {
            },
            StoreOps::Add(e) => {
                t.add(*e);
            },
            StoreOps::Pop => t.pop(),
            StoreOps::IsFullyCovered => {
                t.is_fully_covered();
            },
            StoreOps::UncoveredEdges => {
                size += t.uncovered_edges().min_by_key(|c| c.overlap(t.get_current_cand())).unwrap().len();
            },
            StoreOps::Extend(x) => t.extend(&DefaultEdge::empty(109), x.clone()),
        }
    }
    size
}

fn criterion_benchmark(c: &mut Criterion) {
    let paths = glob::glob("../profiles/*.json").unwrap();


    for path in paths {
        let input : Vec<StoreOps<DefaultEdge>> = serde_json::Deserializer::from_reader(File::open(path.as_ref().unwrap().as_path()).unwrap()).into_iter::<StoreOps<DefaultEdge>>().filter_map(|x| x.ok()).collect();
        c.bench_function(path.as_ref().unwrap().file_name().unwrap().to_str().unwrap(), |b| b.iter(|| bench(black_box(&input))));
    }


}

criterion_group!(benches, criterion_benchmark);
criterion_main!(benches);