use fixedbitset::FixedBitSet;

pub fn set_to(one: &mut FixedBitSet, other: &FixedBitSet) {
    one.grow(other.len());
    for (x, y) in one.as_mut_slice().iter_mut().zip(other.as_slice().iter()) {
        *x = *y;
    }
    let mn = std::cmp::min(one.as_slice().len(), other.as_slice().len());
    for wd in &mut one.as_mut_slice()[mn..] {
        *wd = 0;
    }
}


pub fn set_to_not(one: &mut FixedBitSet, other: &FixedBitSet) {
    one.grow(other.len());
    for (x, y) in one.as_mut_slice().iter_mut().zip(other.as_slice().iter()) {
        *x = !*y;
    }
    let mn = std::cmp::min(one.as_slice().len(), other.as_slice().len());
    for wd in &mut one.as_mut_slice()[mn..] {
        *wd = 0;
    }
}

pub fn intersect_with_inverse(one: &mut FixedBitSet, other: &FixedBitSet) {
    for (x, y) in one.as_mut_slice().iter_mut().zip(other.as_slice().iter()) {
        *x &= !*y;
    }
    let mn = std::cmp::min(one.as_slice().len(), other.as_slice().len());
    for wd in &mut one.as_mut_slice()[mn..] {
        *wd = 0;
    }
}

pub fn is_subset_inverse(one: &FixedBitSet, other: &FixedBitSet) -> bool {
    one.as_slice()
        .iter()
        .zip(other.as_slice().iter())
        .all(|(x, y)| x & y == 0)
        && one.as_slice().iter().skip(other.as_slice().len()).all(|x| *x == 0)
}

pub(crate) fn get_intersection_size(one: &FixedBitSet, other: &FixedBitSet) -> usize {
    one.as_slice()
        .iter()
        .zip(other.as_slice().iter())
        .map(|(x, y)| (x & y).count_ones() as usize)
        .sum()
}