use fixedbitset::FixedBitSet;
use crate::bitutil::set_to;

pub(crate) struct Edgemark(FixedBitSet);

impl Edgemark {
    pub(crate) fn empty(size: usize) -> Edgemark {
        Edgemark(FixedBitSet::with_capacity(size))
    }

    pub(crate) fn filled(size: usize) -> Edgemark {
        let mut res = Edgemark(FixedBitSet::with_capacity(size));
        res.0.set_range(0..size, true);
        res
    }

    pub(crate) fn is_empty(&self) -> bool {
        self.0.as_slice().iter().all(|c| *c == 0)
    }

    pub(crate) fn set(&mut self, idx: usize) {
        self.0.set(idx, true);
    }

    pub(crate) fn is_subset(&self, other: &Edgemark) -> bool {
        self.0.is_subset(&other.0)
    }

    pub(crate) fn grow(&mut self, size: usize) {
        self.0.grow(size);
    }

    pub(crate) fn ones(&self) -> fixedbitset::Ones {
        self.0.ones()
    }
}

impl Clone for Edgemark {
    fn clone(&self) -> Self {
        Edgemark(self.0.clone())
    }

    fn clone_from(&mut self, source: &Self)  {
        set_to(&mut self.0,&source.0);
    }
}

impl std::ops::SubAssign<&Edgemark> for Edgemark {
    fn sub_assign(&mut self, rhs: &Edgemark) {
        self.0.difference_with(&rhs.0);
    }
}

impl std::ops::BitOrAssign<&Edgemark> for Edgemark {
    fn bitor_assign(&mut self, rhs: &Edgemark) {
        self.0 |= &rhs.0;
    }
}

impl std::ops::BitAndAssign<&Edgemark> for Edgemark {
    fn bitand_assign(&mut self, rhs: &Edgemark) {
        self.0 &= &rhs.0;
    }
}
