use std::mem;
use crate::{Cluster, ClusterFilter, ClusterIndex, Pli, RowIndex};

pub(crate) struct RefinementHelper {
    refinement_table: Pli,
    active: Vec<ClusterIndex>,
}

impl RefinementHelper {
    pub(crate) fn new() -> RefinementHelper {
        RefinementHelper {
            refinement_table: vec![Cluster::new(); 0],
            active: Vec::new(),
        }
    }

    pub(crate) fn reserve(&mut self, size: usize) {
        if self.refinement_table.len() < size {
            self.refinement_table.resize(size, Cluster::new());
            self.active.reserve(size - self.active.len());
        }
    }

    pub(crate) fn add(&mut self, clusterid: ClusterIndex, row: RowIndex) {
        let refined_cluster = &mut self.refinement_table[clusterid as usize];
        if refined_cluster.is_empty() {
            self.active.push(clusterid);
        }
        refined_cluster.push(row);
    }

    pub(crate) fn add_refined_cluster(&mut self, target: &mut Vec<Cluster>, filter: &mut impl ClusterFilter) {
        for clusterid in self.active.drain(..) {
            let new_cluster = &mut self.refinement_table[clusterid as usize];
            if filter.keep(new_cluster) {
                target.push(mem::take(new_cluster));
            } else {
                new_cluster.clear();
            }
        }
    }

    pub(crate) fn update_cluster(&mut self, target: &mut Cluster) {
        let mut i = 0;
        for clusterid in self.active.drain(..) {
            let rows = &mut self.refinement_table[clusterid as usize];
            for row in rows.drain(..) {
                target[i] = row;
                i += 1;
            }
        }
    }
}
